using System;
using System.IO;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;

namespace FlvComposer
{
    //=====================================================================================//
    //                                  class FlvCompiler                                  //
    //=====================================================================================//
    class FlvCompiler : IDisposable
    {
        private int m_width = 0;
        private int m_height = 0;
        private FileStream m_file = null;
        private IntPtr m_hFlvStream = IntPtr.Zero;
        private IntPtr m_hScreenVideoEncoder = IntPtr.Zero;
        private NativeMethods.FTWRITEDATACALLBACK m_ftWriteDataCallback = null;

        //=====================================================================================//
        //                                public FlvCompiler()                                 //
        //=====================================================================================//
        public FlvCompiler(string fileName)
        {
            m_file = new FileStream(fileName, FileMode.Create);
            m_ftWriteDataCallback = new NativeMethods.FTWRITEDATACALLBACK(WriteToStreamCallback);
        }

        //=====================================================================================//
        //                               public bool AddFrame()                                //
        //=====================================================================================//
        /// add frame to flv stream
        public bool AddFrame(string imageFileName, int timestamp)
        {
            try
            {
                Bitmap bitmap = new Bitmap(imageFileName);

                if (bitmap.PixelFormat == PixelFormat.Format24bppRgb)
                {
                    if (CheckFlvStream(bitmap.Width, bitmap.Height))
                    {
                        if (m_hScreenVideoEncoder != IntPtr.Zero)
                        {
                            // lock the bitmap's bits.  
                            Rectangle bmRect = new Rectangle(0, 0, bitmap.Width, bitmap.Height);
                            BitmapData bmData = bitmap.LockBits(bmRect, ImageLockMode.ReadOnly, bitmap.PixelFormat);

                            // get the address of the first line.
                            IntPtr bmDataPtr = bmData.Scan0;
                            Int32 bmDataSize = bmData.Height * bmData.Stride;

                            //  encode the video frame
                            NativeMethods.ftScreenVideoEncodeFrame(m_hScreenVideoEncoder, bmDataPtr, bmDataSize, false);

                            //  unlock the bitmap's bits.  
                            bitmap.UnlockBits(bmData);

                            //  get the encoded video frame
                            IntPtr lpData = IntPtr.Zero;
                            Int32 cbBytes = 0;
                            NativeMethods.ftScreenVideoGetEncodedFrame(m_hScreenVideoEncoder, ref lpData, ref cbBytes);

                            //  add frame to Flv stream
                            NativeMethods.ftFlvStreamPutVideoData(m_hFlvStream, lpData, cbBytes, timestamp);

                            return true;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Flash Video Composer", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

	        return false;
        }

        //=====================================================================================//
        //                                public void Dispose()                                //
        //=====================================================================================//
        /// implements IDisposable
        public void Dispose()
        {
            //  release Screen Video encoder
            if (m_hScreenVideoEncoder != IntPtr.Zero)
            {
                NativeMethods.ftScreenVideoEncoderDestroy(m_hScreenVideoEncoder);
                m_hScreenVideoEncoder = IntPtr.Zero;
            }
            //  release Flv Stream
            if (m_hFlvStream != IntPtr.Zero)
            {
                NativeMethods.ftFlvStreamDestroy(m_hFlvStream);
                m_hFlvStream = IntPtr.Zero;
            }
            //  close output file
            m_file.Close();
        }

        //=====================================================================================//
        //                            private bool CheckFlvStream()                            //
        //=====================================================================================//
        ///	check flv stream state and parameters
        private bool CheckFlvStream(int width, int height)
        {
            if(m_hFlvStream == IntPtr.Zero)
	        {
                //	initialize FTVIDEOINFO struct
                NativeMethods.FTVIDEOINFO ftvi = new NativeMethods.FTVIDEOINFO();
                ftvi.dwCodec = NativeMethods.FT_VIDEOCODEC_SCREENVIDEO;
                ftvi.dwWidth = width;
                ftvi.dwHeight = height;

                //	initialize FTFLVSTREAMINFO struct
                GCHandle gchFtvi = GCHandle.Alloc(ftvi, GCHandleType.Pinned);
                NativeMethods.FTFLVSTREAMINFO ftflvsi = new NativeMethods.FTFLVSTREAMINFO();
                ftflvsi.lpVideoInfo = gchFtvi.AddrOfPinnedObject();
                ftflvsi.lpAudioInfo = IntPtr.Zero;
                ftflvsi.lpWriteDataCallback = m_ftWriteDataCallback;
                ftflvsi.lpParam = IntPtr.Zero;

                //	create flv stream
                m_hFlvStream = NativeMethods.ftFlvStreamCreate(ref ftflvsi);
                gchFtvi.Free();

                //	create Screen Video encoder
                const int block_size = 16;
                m_hScreenVideoEncoder = NativeMethods.ftScreenVideoEncoderCreate(width,height,block_size);

                m_width = width;
                m_height = height;
	        }

            return (m_hFlvStream != IntPtr.Zero) && (m_width == width) && (m_height == height);
        }

        //=====================================================================================//
        //                        private void WriteToStreamCallback()                         //
        //=====================================================================================//
        private void WriteToStreamCallback(IntPtr lpParam, IntPtr lpData, Int32 nLen)
        {
            if (nLen != 0)
            {
                byte[] buffer = new byte[nLen];
                Marshal.Copy(lpData, buffer, 0, nLen);
                m_file.Write(buffer, 0, nLen);
            }
        }
    }
}
